# acuvity

Developer-friendly & type-safe Python SDK specifically catered to leverage the Acuvity APIs - in particularly the Apex API.

<div align="left">
    <a href="https://www.apache.org/licenses/LICENSE-2.0.html">
        <img src="https://img.shields.io/badge/License-Apache_2.0-blue.svg" style="width: 100px; height: 28px;" />
    </a>
</div>

<!-- Start Summary [summary] -->
## Summary

Apex API: Acuvity Apex provides access to scan and detection APIs
<!-- End Summary [summary] -->

<!-- Start Table of Contents [toc] -->
## Table of Contents
<!-- $toc-max-depth=2 -->
* [acuvity](#acuvity)
  * [SDK Installation](#sdk-installation)
  * [IDE Support](#ide-support)
  * [SDK Example Usage](#sdk-example-usage)
  * [Available Resources and Operations](#available-resources-and-operations)
  * [Retries](#retries)
  * [Error Handling](#error-handling)
  * [Server Selection](#server-selection)
  * [Custom HTTP Client](#custom-http-client)
  * [Authentication](#authentication)
  * [Debugging](#debugging)
* [Development](#development)
  * [Maturity](#maturity)
  * [Contributions](#contributions)

<!-- End Table of Contents [toc] -->

<!-- Start SDK Installation [installation] -->
## SDK Installation

The SDK can be installed with either *pip* or *poetry* package managers.

### PIP

*PIP* is the default package installer for Python, enabling easy installation and management of packages from PyPI via the command line.

```bash
pip install acuvity
```

### Poetry

*Poetry* is a modern tool that simplifies dependency management and package publishing by using a single `pyproject.toml` file to handle project metadata and dependencies.

```bash
poetry add acuvity
```
<!-- End SDK Installation [installation] -->

<!-- Start IDE Support [idesupport] -->
## IDE Support

### PyCharm

Generally, the SDK will work well with most IDEs out of the box. However, when using PyCharm, you can enjoy much better integration with Pydantic by installing an additional plugin.

- [PyCharm Pydantic Plugin](https://docs.pydantic.dev/latest/integrations/pycharm/)
<!-- End IDE Support [idesupport] -->

<!-- No SDK Example Usage [usage] -->
## SDK Example Usage

### Process a scan request

For the most simple example of using the Scan API, ensure that you have your app token set in the `ACUVITY_TOKEN` environment variable.
You can then run the following:

```python
from acuvity import Acuvity

c = Acuvity()
res = c.apex.scan("My prompt I want to scan")
print(res)
```

Here is a more elaborate scan request using the Scan API and making use of the builtin context manager.

```python
# Synchronous Example
import acuvity
from acuvity import Acuvity
import os

with Acuvity(
    security=acuvity.Security(
        # this is the default and can be omitted
        token=os.getenv("ACUVITY_TOKEN", ""),
    ),
) as acuvity:

    res = acuvity.apex.scan(
        "Using a weather forecasting service, provide me with a weather forecast for the next ten days for Sunnyvale, CA."
    )

    if res is not None:
        # handle response
        pass
```

</br>

The same SDK client can also be used to make asychronous requests by importing asyncio.
```python
# Asynchronous Example
import acuvity
from acuvity import Acuvity
import asyncio
import os

async def main():
    async with Acuvity(
        security=acuvity.Security(
            # this is the default and can be omitted
            token=os.getenv("ACUVITY_TOKEN", ""),
        ),
    ) as acuvity:

        res = await acuvity.apex.scan_async(
            "Using a weather forecasting service, provide me with a weather forecast for the next ten days for Sunnyvale, CA."
        )

        if res is not None:
            # handle response
            pass

asyncio.run(main())
```

### List all available analyzers

Now you can list all available analyzers that can be used in the Scan API.

```python
# Synchronous Example
import acuvity
from acuvity import Acuvity
import os

with Acuvity(
    security=acuvity.Security(
        # this is the default and can be omitted
        token=os.getenv("ACUVITY_TOKEN", ""),
    ),
) as acuvity:

    res = acuvity.apex.list_analyzers()

    if res is not None:
        # handle response
        pass
```

</br>

The same SDK client can also be used to make asychronous requests by importing asyncio.
```python
# Asynchronous Example
import acuvity
from acuvity import Acuvity
import asyncio
import os

async def main():
    async with Acuvity(
        security=acuvity.Security(
            # this is the default and can be omitted
            token=os.getenv("ACUVITY_TOKEN", ""),
        ),
    ) as acuvity:

        res = await acuvity.apex.list_analyzers_async()

        if res is not None:
            # handle response
            pass

asyncio.run(main())
```

**NOTE:** If you simply want to get a list of analyzer names or groups that can be used in the scan API, use `list_analyzer_names()` or `list_analyzer_groups()` instead.
<!-- No SDK Example Usage [usage] -->

<!-- Start Available Resources and Operations [operations] -->
## Available Resources and Operations

<details open>
<summary>Available methods</summary>


### [apex](docs/sdks/apex/README.md)

* [list_analyzers](docs/sdks/apex/README.md#list_analyzers) - List of all available analyzers.
* [police_request](docs/sdks/apex/README.md#police_request) - Processes the scan and police request.
* [scan_request](docs/sdks/apex/README.md#scan_request) - Processes the scan request.

</details>
<!-- End Available Resources and Operations [operations] -->

<!-- Start Retries [retries] -->
## Retries

Some of the endpoints in this SDK support retries. If you use the SDK without any configuration, it will fall back to the default retry strategy provided by the API. However, the default retry strategy can be overridden on a per-operation basis, or across the entire SDK.

To change the default retry strategy for a single API call, simply provide a `RetryConfig` object to the call:
```python
import acuvity
from acuvity import Acuvity
from acuvity.utils import BackoffStrategy, RetryConfig
import os

with Acuvity(
    security=acuvity.Security(
        token=os.getenv("ACUVITY_TOKEN", ""),
    ),
) as acuvity:

    res = acuvity.apex.list_analyzers(,
        RetryConfig("backoff", BackoffStrategy(1, 50, 1.1, 100), False))

    # Handle response
    print(res)

```

If you'd like to override the default retry strategy for all operations that support retries, you can use the `retry_config` optional parameter when initializing the SDK:
```python
import acuvity
from acuvity import Acuvity
from acuvity.utils import BackoffStrategy, RetryConfig
import os

with Acuvity(
    retry_config=RetryConfig("backoff", BackoffStrategy(1, 50, 1.1, 100), False),
    security=acuvity.Security(
        token=os.getenv("ACUVITY_TOKEN", ""),
    ),
) as acuvity:

    res = acuvity.apex.list_analyzers()

    # Handle response
    print(res)

```
<!-- End Retries [retries] -->

<!-- Start Error Handling [errors] -->
## Error Handling

Handling errors in this SDK should largely match your expectations. All operations return a response object or raise an exception.

By default, an API error will raise a models.APIError exception, which has the following properties:

| Property        | Type             | Description           |
|-----------------|------------------|-----------------------|
| `.status_code`  | *int*            | The HTTP status code  |
| `.message`      | *str*            | The error message     |
| `.raw_response` | *httpx.Response* | The raw HTTP response |
| `.body`         | *str*            | The response content  |

When custom error responses are specified for an operation, the SDK may also raise their associated exceptions. You can refer to respective *Errors* tables in SDK docs for more details on possible exception types for each operation. For example, the `list_analyzers_async` method may raise the following exceptions:

| Error Type            | Status Code   | Content Type     |
| --------------------- | ------------- | ---------------- |
| models.Elementalerror | 400, 401, 500 | application/json |
| models.APIError       | 4XX, 5XX      | \*/\*            |

### Example

```python
import acuvity
from acuvity import Acuvity, models
import os

with Acuvity(
    security=acuvity.Security(
        token=os.getenv("ACUVITY_TOKEN", ""),
    ),
) as acuvity:
    res = None
    try:

        res = acuvity.apex.list_analyzers()

        # Handle response
        print(res)

    except models.Elementalerror as e:
        # handle e.data: models.ElementalerrorData
        raise(e)
    except models.APIError as e:
        # handle exception
        raise(e)
```
<!-- End Error Handling [errors] -->

<!-- No Server Selection [server] -->
## Server Selection

### Server Variables

The default server `https://{apex_domain}:{apex_port}` contains variables and is set to `https://apex.acuvity.ai:443` by default. Note that the default values **DO NOT** point to a valid and existing Apex URL as they are specific and unique to every organization. Therefore both variables must be set. The following parameters are available when initializing the SDK client instance:
 * `apex_domain: str`
 * `apex_port: str`

However, if no `server_url` or `apex_domain` is set, the Apex URL is being automatically determined based on the provided token (either inside of the `Security` object or set in the `ACUVITY_TOKEN` environment variable).
Note that this is going to make additional internal (synchronous) API calls to the Acuvity backend to determine the Apex URL.
This is a one-time operation that runs during the initialization of the `Acuvity` class.

### Override Server URL Per-Client

The default server can also be overridden globally by passing a URL to the `server_url: str` optional parameter when initializing the SDK client instance. For example:
```python
import acuvity
from acuvity import Acuvity
import os

with Acuvity(
    server_url="https://apex.acuvity.ai:443",
    security=acuvity.Security(
        token=os.getenv("ACUVITY_TOKEN", ""),
    ),
) as acuvity:

    res = acuvity.apex.list_analyzers()

    if res is not None:
        # handle response
        pass

```
<!-- No Server Selection [server] -->

<!-- Start Custom HTTP Client [http-client] -->
## Custom HTTP Client

The Python SDK makes API calls using the [httpx](https://www.python-httpx.org/) HTTP library.  In order to provide a convenient way to configure timeouts, cookies, proxies, custom headers, and other low-level configuration, you can initialize the SDK client with your own HTTP client instance.
Depending on whether you are using the sync or async version of the SDK, you can pass an instance of `HttpClient` or `AsyncHttpClient` respectively, which are Protocol's ensuring that the client has the necessary methods to make API calls.
This allows you to wrap the client with your own custom logic, such as adding custom headers, logging, or error handling, or you can just pass an instance of `httpx.Client` or `httpx.AsyncClient` directly.

For example, you could specify a header for every request that this sdk makes as follows:
```python
from acuvity import Acuvity
import httpx

http_client = httpx.Client(headers={"x-custom-header": "someValue"})
s = Acuvity(client=http_client)
```

or you could wrap the client with your own custom logic:
```python
from acuvity import Acuvity
from acuvity.httpclient import AsyncHttpClient
import httpx

class CustomClient(AsyncHttpClient):
    client: AsyncHttpClient

    def __init__(self, client: AsyncHttpClient):
        self.client = client

    async def send(
        self,
        request: httpx.Request,
        *,
        stream: bool = False,
        auth: Union[
            httpx._types.AuthTypes, httpx._client.UseClientDefault, None
        ] = httpx.USE_CLIENT_DEFAULT,
        follow_redirects: Union[
            bool, httpx._client.UseClientDefault
        ] = httpx.USE_CLIENT_DEFAULT,
    ) -> httpx.Response:
        request.headers["Client-Level-Header"] = "added by client"

        return await self.client.send(
            request, stream=stream, auth=auth, follow_redirects=follow_redirects
        )

    def build_request(
        self,
        method: str,
        url: httpx._types.URLTypes,
        *,
        content: Optional[httpx._types.RequestContent] = None,
        data: Optional[httpx._types.RequestData] = None,
        files: Optional[httpx._types.RequestFiles] = None,
        json: Optional[Any] = None,
        params: Optional[httpx._types.QueryParamTypes] = None,
        headers: Optional[httpx._types.HeaderTypes] = None,
        cookies: Optional[httpx._types.CookieTypes] = None,
        timeout: Union[
            httpx._types.TimeoutTypes, httpx._client.UseClientDefault
        ] = httpx.USE_CLIENT_DEFAULT,
        extensions: Optional[httpx._types.RequestExtensions] = None,
    ) -> httpx.Request:
        return self.client.build_request(
            method,
            url,
            content=content,
            data=data,
            files=files,
            json=json,
            params=params,
            headers=headers,
            cookies=cookies,
            timeout=timeout,
            extensions=extensions,
        )

s = Acuvity(async_client=CustomClient(httpx.AsyncClient()))
```
<!-- End Custom HTTP Client [http-client] -->

<!-- Start Authentication [security] -->
## Authentication

### Per-Client Security Schemes

This SDK supports the following security schemes globally:

| Name     | Type   | Scheme      | Environment Variable |
| -------- | ------ | ----------- | -------------------- |
| `token`  | http   | HTTP Bearer | `ACUVITY_TOKEN`      |
| `cookie` | apiKey | API key     | `ACUVITY_COOKIE`     |

You can set the security parameters through the `security` optional parameter when initializing the SDK client instance. The selected scheme will be used by default to authenticate with the API for all operations that support it. For example:
```python
import acuvity
from acuvity import Acuvity
import os

with Acuvity(
    security=acuvity.Security(
        token=os.getenv("ACUVITY_TOKEN", ""),
    ),
) as acuvity:

    res = acuvity.apex.list_analyzers()

    # Handle response
    print(res)

```
<!-- End Authentication [security] -->

<!-- Start Debugging [debug] -->
## Debugging

You can setup your SDK to emit debug logs for SDK requests and responses.

You can pass your own logger class directly into your SDK.
```python
from acuvity import Acuvity
import logging

logging.basicConfig(level=logging.DEBUG)
s = Acuvity(debug_logger=logging.getLogger("acuvity"))
```

You can also enable a default debug logger by setting an environment variable `ACUVITY_DEBUG` to true.
<!-- End Debugging [debug] -->

<!-- Placeholder for Future Speakeasy SDK Sections -->

# Development

## Maturity

This SDK is in beta, and there may be breaking changes between versions without a major version update. Therefore, we recommend pinning usage
to a specific package version. This way, you can install the same version each time without breaking changes unless you are intentionally
looking for the latest version.

## Contributions

While we value open-source contributions to this SDK, this library is generated programmatically. Any manual changes added to internal files will be overwritten on the next generation. 
We look forward to hearing your feedback. Feel free to open a PR or an issue with a proof of concept and we'll do our best to include it in a future release. 
